/*
  rlp-udp.cpp

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "rlp-udp.h"
#include <cstring>

namespace ACN {
namespace RLP {
namespace UDP {

preamble_t::preamble_t(PDU::Stream stream) {
  length = stream->read16();
  postamble_size = stream->read16();
  stream->read(acn_id, 12);
  if (stream->gcount() != 12)
    stream->setstate(stream->rdstate() | std::ios_base::failbit);
}


/*
  Validate compliance with ACN EPI 17: ACN Root Layer Protocol on UDP
  4.2. Reception
*/
preamble_t::operator bool () {
  // 2. Preamble Format: The preamble size includes both size fields so the
  // minimum value for preamble size is 16 (octets).
  if (length < PREAMBLE_MINIMUM_SIZE)
    return false;

  // 2. Preamble Format: The ACN Packet Identifier shall be the text string
  // “ASC-E1.17\0\0\0” encoded in [ASCII].
  if (memcmp(acn_id, ACN_PACKET_IDENTIFIER, 12))
    return false;

  return true;
}


} // UDP
} // RLP
} // ACN
