/*
  rlp.cpp

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "rlp.h"

namespace ACN {
namespace RLP {

rlp_header::rlp_header(PDU::Stream stream)
  : PDU::pdu_header()
  , cid()
{
  uint8_t buf[16];
  stream->read(buf, 16);
  if (stream->gcount() != 16)
    stream->setstate(stream->rdstate() | std::ios_base::failbit);
  cid = UUID::uuid(buf);
}

Pdu::Pdu(PDU::Stream stream)
  : ACN::PDU::Pdu(stream, 4)
{
  if (stream->fail()) return;
  if (!buffer_->good()) return;

  if (flags_.hasHeader)
    setHeader(new rlp_header(buffer_));
}

} // RLP
} // ACN
