/*
  sdt-udp.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>
#include "pdu.h"

// ACN EPI 18 - Operation of SDT on UDP Networks
namespace ACN {
namespace SDT {
namespace UDP {

//  Table 3. SDT symbolic parameters
static const float    MAK_TIMEOUT_FACTOR   = 0.1;
static const uint8_t  MAK_MAX_RETRIES      = 2;    // (3 tries total)
static const uint8_t  AD_HOC_TIMEOUT       = 200;  // ms
static const uint8_t  AD_HOC_RETRIES       = 2;    // (3 tries total)
static const float    RECIPROCAL_TIMEOUT_FACTOR = 0.2;
static const uint8_t  MIN_EXPIRY_TIME      = 2;    // s
static const float    NAK_TIMEOUT_FACTOR   = 0.1;
static const uint8_t  NAK_MAX_RETRIES      = 2;    // (3 tries total)
static const uint8_t  NAK_HOLDOFF_INTERVAL = 2;    // ms
static const uint8_t  NAK_MAX_TIME         = 10 * NAK_HOLDOFF_INTERVAL;
static const uint8_t  NAK_BLANKTIME        = 3 * NAK_HOLDOFF_INTERVAL;
static const uint16_t SDT_MULTICAST_PORT   = 5568; // IANA registered “sdt”

// defined in SDT, but needed here without including that header
enum ip_addr_spec_t {
  SDT_ADDR_NULL = 0,  // Address is not present (0 octets).
  SDT_ADDR_IPV4 = 1,  // Address specified is in IP v4 format
  SDT_ADDR_IPV6 = 2   // Address specified is in IP v6 format
};

// 3 Address Specification
struct address_t {
  uint8_t  type       = SDT_ADDR_NULL;
  uint16_t port       = SDT_MULTICAST_PORT;
  union {
    uint8_t  ipv4[4];
    uint8_t  ipv6[16] = {0};
  };
  address_t() {};
  address_t(PDU::Stream);
};

} // UDP
} // SDT
} // ACN
