/*
  universe.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>
#include <functional>
#include <vector>

namespace DMX {

using std::uint8_t;
using std::uint16_t;
using std::array;
using std::function;
using std::vector;

// Table D1 - Reserved START Codes
static const uint8_t E111_NULL_START       = 0;
static const uint8_t E111_ASC_TEXT_ASCII   = 23;
static const uint8_t E111_ASC_TEST         = 85;
static const uint8_t E111_ASC_TEXT_UTF8    = 144;
static const uint8_t E111_ASC_MANUFACTURER = 145;
static const uint8_t E111_ASC_SIP          = 207;


class Universe; // forward declare the Univserse class
using DataHandler = function<void(Universe *)>;
using DimmerData  = array<uint8_t, 513>;

/*
   The Universe class

   ANSI E1.11 describes may things about moving data over
   serial EIA-485-A links.  This class encapselates that data,
   regardless of how it was transmitted.
*/
class Universe {
  public:
    Universe ();

    DimmerData * data() { return &null_start_data_; }
    void    onData (const DataHandler callback);
    void    set    (vector<uint8_t>);
    uint8_t slot   (uint16_t address) { return null_start_data_[address]; }

  private:
    DimmerData null_start_data_;
    vector<DataHandler> callbacks_;
};

} // DMX
