/*
  receiver-esp.cpp

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "sacn-receiver.h"

// ESP logging module
#include <Arduino.h>
#include <esp_log.h>
static const char* TAG = "EspReceiver";

namespace SACN {

EspReceiver::EspReceiver(UUID::uuid cid)
  : Receiver(cid)
{
  udp.onPacket(std::bind(&EspReceiver::UdpStreamHandler, this,
                         std::placeholders::_1));
}

void EspReceiver::subscribe(const uint16_t num) {
  if (universe(num))  // already subscribed
    return;
  ESP_LOGI(TAG, "Subscribing to universe # %d", num);
  Receiver::subscribe(num);
  // listen multicast; works for unicast too
  udp.listenMulticast(IPv4MulticastAddress(num),
                      ACN_SDT_MULTICAST_PORT);
}

void EspReceiver::unsubscribe(const uint16_t num) {
  if (!universe(num)) // not subscribed
    return;
  ESP_LOGI(TAG, "Unubscribing from universe # %d", num);
  Receiver::unsubscribe(num);
  // AsyncUDP has no way to unsubscribe IGMP?
}

void EspReceiver::UdpStreamHandler(AsyncUDPPacket udp_packet) {
  // Expecting IANA registered Session Data Transport traffic
  if (!udp_packet.localPort() == ACN_SDT_MULTICAST_PORT)
    return;

  // wrap a PDU io stream around the AsyncUDPPacket data buffer
  PDU::Stream stream(new PDU::pdu_stream(udp_packet.data(),
                                         udp_packet.available()));
  Appliance::UdpStreamHandler(stream);
}

} // SACN
