/*
  receiver-esp.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <Arduino.h>
#include <IPAddress.h>
#include <IPv6Address.h>
#include <AsyncUDP.h>
#include "../../../sacn/receiver.h"
#include "../../../dmx/universe.h"


/*
  Arduino specification additions to the sACN library
*/
namespace SACN {
  // 9.3.1 Allocation of IPv4 Multicast Addresses
  // Multicast addresses are from the IPv4 Local Scope.
  inline IPAddress IPv4MulticastAddress(uint16_t universe) {
      return IPAddress(239, 255, (universe >> 8), (universe & 0xff));
  };

  // 9.3.2 Allocation of IPv6 Multicast Addresses
  inline IPv6Address IPv6MulticastAddress(uint16_t universe) {
      IPv6Address address;
      address.fromString("ff18::83:00:" +
                         String(universe >> 8, HEX) + ":" +
                         String(universe & 0xff, HEX));
      return address;
  };

class EspReceiver
  : public SACN::Receiver
{
  public:
    EspReceiver(UUID::uuid = UUID::uuid());
    virtual void subscribe(const uint16_t universe = 1);
    virtual void unsubscribe(const uint16_t);

  private:
    AsyncUDP   udp;      // AsyncUDP
    void UdpStreamHandler(AsyncUDPPacket); // UDP packet parser callback
};

} // SACN
