/*
  data.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "sacn.h"

namespace SACN {
namespace DATA {

using std::uint8_t;
using std::uint16_t;

// Table 6-1: E1.31 Data Packet Framing Layer
struct  frame_header : PDU::pdu_header {
  uint8_t  source_name[64];
  uint8_t  priority;
  uint16_t sync_address;
  uint8_t  sequence_number;
  uint8_t  options;
  uint16_t universe;
  frame_header(PDU::Stream);
};

// 6.2.6 E1.31 Data Packet: Options
enum options_t : uint8_t {
  PREVIEW_DATA          = 0b10000000, // Bit 7 = Preview_Data
  STREAM_TERMINATED     = 0b01000000, // Bit 6 = Stream_Terminated
  FORCE_SYNCHRONIZATION = 0b00100000, // Bit 5 = Force_Synchronization
};

class Pdu
  : public PDU::Pdu
{
public:
  Pdu(PDU::Stream);
};

} // DATA
} // SACN
