/*
  sacn.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>
#include "../acn/acn.h"

// E1.31 Lightweight streaming protocol for transport of DMX512 using ACN
namespace SACN {

using namespace ACN;
using std::uint8_t;
using std::uint16_t;
using std::uint32_t;

// Appendix A: Defined Parameters (Normative)
static const uint32_t VECTOR_ROOT_E131_DATA                = 0x00000004;
static const uint32_t VECTOR_ROOT_E131_EXTENDED            = 0x00000008;

static const uint8_t  VECTOR_DMP_SET_PROPERTY              = 0x02;

static const uint32_t VECTOR_E131_DATA_PACKET              = 0x00000002;

static const uint32_t VECTOR_E131_EXTENDED_SYNCHRONIZATION = 0x00000001;
static const uint32_t VECTOR_E131_EXTENDED_DISCOVERY       = 0x00000002;

static const uint32_t VECTOR_UNIVERSE_DISCOVERY_UNIVERSE_LIST = 0x00000001;

static const uint16_t E131_E131_UNIVERSE_DISCOVER_INTERVAL = 10000; // ms
static const uint16_t E131_NETWORK_DATA_LOSS_TIMEOUT       = 2500; // ms
static const uint16_t E131_DISCOVERY_UNIVERSE              = 64214;

static const uint16_t ACN_SDT_MULTICAST_PORT               = 5568;
} // SACN
