/*
  universe.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "sacn.h"
#include "data.h"
#include "../dmx/universe.h"
#include "../uuid/uuid.h"
#include <memory>
#include <string>

namespace SACN {

/**
  universe metadata
*/
class UniverseSource
{
public:
  UniverseSource(std::shared_ptr<DATA::Pdu>);

  const UUID::uuid  cid()          const {return cid_;};
  const std::string description()  const {return description_;}
  const uint16_t    universe()     const {return universe_;}
  const uint8_t     priority()     const {return priority_;}
  const uint16_t    syncAddress()  const {return sync_address_;}
  const bool        isTerminated() const {return options_
                                                & DATA::STREAM_TERMINATED;}
  const bool        isPreview()    const {return options_
                                                & DATA::PREVIEW_DATA;}
  const bool        isForced()     const {return options_
                                                & DATA::FORCE_SYNCHRONIZATION;}
private:
  UUID::uuid  cid_;
  std::string description_;
  uint16_t    universe_;
  uint8_t     priority_;
  uint16_t    sync_address_;
  uint8_t     options_;
};


/**
  universe data
*/
class Universe
  : public DMX::Universe
{
public:
  Universe();
  bool isSyncronized() const {return synchronized_;};

  void set(std::shared_ptr<DMP::Pdu>, std::shared_ptr<UniverseSource>);

private:
  bool synchronized_;
  std::shared_ptr<UniverseSource> source_;
};


};
